(ns actuarial.mixed-distributions (:use actuarial.distributions))

(defn mixedDist
  [prob-dist cont-dist]
  {:Distribution ::Mixed
   :SubDistributions {:prob-dist prob-dist
                      :cont-dist cont-dist}
   :type ::Mixed})

(defmacro getmixed [f m & params]
  `(let [probs# (get-in ~m [:SubDistributions :prob-dist :Parameters :prob])
         dists# (get-in ~m [:SubDistributions :cont-dist])]
     (reduce + (map #(* %1 (~f %2 ~@params)) probs# dists#))))

(defmethod pdf ::Mixed [m x]
  (getmixed pdf m x))

(defmethod cdf ::Mixed [m x]
  (getmixed cdf m x))

(defmethod mgf ::Mixed [m t]
  (getmixed mgf m t))

(defmethod rawmoments ::Mixed [m k]
  (getmixed rawmoments m k))

(defmethod truncated-mean ::Mixed [m d]
  (getmixed truncated-mean m d))




